# 两图水平拼接
import cv2
import numpy as np


def getHorizen(image1, image2):
  hh, ww = image1.shape[:2]
  hh1, ww1 = image2.shape[:2]

  h=max(hh,hh1)
  w=max(ww,ww1)
  org_image=np.ones((h,w,3),dtype=np.uint8)*255
  trans_image=np.ones((h,w,3),dtype=np.uint8)*255

  org_image[:hh,:ww,:]=image1[:,:,:]
  trans_image[:hh1,:ww1,:]=image2[:,:,:]
  all_image = np.hstack((org_image[:,:ww,:], trans_image[:,:ww1,:]))

  # save images
  cv2.imwrite('horizen.jpg', all_image)



# 两图相减法
import cv2
import numpy as np


def getOverlapping(image1, image2):
  hh, ww = image1.shape[:2]
  
  hh1, ww1 = image2.shape[:2]

  h=max(hh,hh1)
  w=max(ww,ww1)
  org_image=np.ones((h,w,3),dtype=np.uint8)*255
  trans_image=np.ones((h,w,3),dtype=np.uint8)*255

  org_image[:hh,:ww,:]=image1[:,:,:]
  trans_image[:hh1,:ww1,:]=image2[:,:,:]

  # substract_img = trans_image - org_image
  # 权重越大，透明度越低
  overlapping = cv2.addWeighted(org_image, 0.7, trans_image, 0.3, 0)
  # 保存叠加后的图片
  cv2.imwrite('overlapping.jpg', overlapping)





import cv2
import numpy as np

# 最大要素数量 默认500
MAX_FEATURES = 500
GOOD_MATCH_PERCENT = 0.15

def alignImages(im1, im2):

  # im2 is reference and im1 is to be warped to match im2
  # note: numbering is swapped in function
 
  # Convert images to grayscale
  im1Gray = cv2.cvtColor(im1, cv2.COLOR_BGR2GRAY)
  im2Gray = cv2.cvtColor(im2, cv2.COLOR_BGR2GRAY)
   
  # Detect ORB features and compute descriptors.
  # orb 算法
  orb = cv2.ORB_create(MAX_FEATURES)
  keypoints1, descriptors1 = orb.detectAndCompute(im1Gray, None)
  keypoints2, descriptors2 = orb.detectAndCompute(im2Gray, None)
   
  # Match features.
  matcher = cv2.DescriptorMatcher_create(cv2.DESCRIPTOR_MATCHER_BRUTEFORCE_HAMMING)
  matches = matcher.match(descriptors1, descriptors2, None)
   
  # Sort matches by score
  # matches.sort(key=lambda x: x.distance, reverse=False)
  matches = sorted(matches, key=lambda x: x.distance, reverse=False)
  # Remove not so good matches
  numGoodMatches = int(len(matches) * GOOD_MATCH_PERCENT)
  matches = matches[:numGoodMatches]
 
  # Draw top matches
  imMatches = cv2.drawMatches(im1, keypoints1, im2, keypoints2, matches, None)
  cv2.imwrite("diff_matches.png", imMatches)
   
  # Extract location of good matches
  points1 = np.zeros((len(matches), 2), dtype=np.float32)
  points2 = np.zeros((len(matches), 2), dtype=np.float32)
 
  for i, match in enumerate(matches):
    points1[i, :] = keypoints1[match.queryIdx].pt
    points2[i, :] = keypoints2[match.trainIdx].pt
   
  # Find homography
  h, mask = cv2.findHomography(points1, points2, cv2.RANSAC)
 
  # Use homography
  height, width, channels = im2.shape
  im1Reg = cv2.warpPerspective(im1, h, (width, height))
   
  return im1Reg, h
 
 
def getDiffImages(image1, image2):
  hh, ww = image1.shape[:2]
  
  # Aligned image will be stored in imReg. 
  # The estimated homography will be stored in h. 
  imReg, h = alignImages(image2, image1)
   
  # Print estimated homography
  # print("Estimated homography : \n",  h)
  
  # Convert images to HSV and get saturation channel
  refSat = cv2.cvtColor(image1, cv2.COLOR_BGR2HSV)[:,:,1]
  imSat = cv2.cvtColor(imReg, cv2.COLOR_BGR2HSV)[:,:,1]

  # Otsu threshold
  refThresh = cv2.threshold(refSat, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)[1]
  imThresh = cv2.threshold(imSat, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)[1]

  # apply morphology open and close
  kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (7,7))
  refThresh = cv2.morphologyEx(refThresh, cv2.MORPH_OPEN, kernel, iterations=1)
  refThresh = cv2.morphologyEx(refThresh, cv2.MORPH_CLOSE, kernel, iterations=1).astype(np.float64)
  imThresh = cv2.morphologyEx(imThresh, cv2.MORPH_OPEN, kernel, iterations=1).astype(np.float64)
  imThresh = cv2.morphologyEx(imThresh, cv2.MORPH_CLOSE, kernel, iterations=1)
  
  # get absolute difference between the two thresholded images
  diff = np.abs(cv2.add(imThresh,-refThresh))
  
  # apply morphology open to remove small regions caused by slight misalignment of the two images
  kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (12,12))
  diff_cleaned = cv2.morphologyEx(diff, cv2.MORPH_OPEN, kernel, iterations=1).astype(np.uint8)

  # Filter using contour area and draw bounding boxes that do not touch the sides of the image
  cnts = cv2.findContours(diff_cleaned, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
  cnts = cnts[0] if len(cnts) == 2 else cnts[1]
  result = image1.copy()
  for c in cnts:
      x,y,w,h = cv2.boundingRect(c)
      if x>0 and y>0 and x+w<ww-1 and y+h<hh-1:
        cv2.rectangle(result, (x, y), (x+w, y+h), (0, 0, 255), 2)

  # save images
  cv2.imwrite('diff.png', diff_cleaned)
  cv2.imwrite('diff_result.png', result)





#!/usr/bin/env python3
import sys
import os
import cv2
import numpy as np

from horizen import getHorizen
from substract import getSubstract
from overlapping import getOverlapping
from diff import getDiffImages
from hist_compare import getHistResult

def main():
  curPath = os.path.abspath('.')
  im1Path = os.path.join(curPath, '1.jpg')
  im2Path = os.path.join(curPath, '2.jpg')
  image1 = cv2.imread(im1Path)
  image2 = cv2.imread(im2Path)

  getHorizen(image1, image2)
  getSubstract(image1, image2)
  getOverlapping(image1, image2)
  getDiffImages(image1, image2)
  getHistResult(image1, image2)


if __name__ == '__main__':
  main()



import cv2 as cv
import numpy as np
from matplotlib import pyplot as plt


def create_rgb_hist(image):
  """"创建 RGB 三通道直方图（直方图矩阵）"""
  h, w, c = image.shape
  # 创建一个（16*16*16,1）的初始矩阵，作为直方图矩阵 
  # 16*16*16的意思为三通道每通道有16个bins
  rgbhist = np.zeros([16 * 16 * 16, 1], np.float32)
  bsize = 256 / 16
  for row in range(h):
    for col in range(w):
      b = image[row, col, 0]
      g = image[row, col, 1]
      r = image[row, col, 2]
      # 人为构建直方图矩阵的索引，该索引是通过每一个像素点的三通道值进行构建
      index = int(b / bsize) * 16 * 16 + int(g / bsize) * 16 + int(r / bsize)
      # 该处形成的矩阵即为直方图矩阵
      rgbhist[int(index), 0] += 1
  # plt.ylim([0, 10000])
  # plt.grid(color='r', linestyle='--', linewidth=0.5, alpha=0.3)
  return rgbhist

def hist_compare(image1, image2):
  """直方图比较函数"""
  # 创建第一幅图的rgb三通道直方图（直方图矩阵）
  hist1 = create_rgb_hist(image1)
  # 创建第二幅图的rgb三通道直方图（直方图矩阵）
  hist2 = create_rgb_hist(image2)
  # 进行三种方式的直方图比较
  match1 = cv.compareHist(hist1, hist2, cv.HISTCMP_BHATTACHARYYA)
  match2 = cv.compareHist(hist1, hist2, cv.HISTCMP_CORREL)
  match3 = cv.compareHist(hist1, hist2, cv.HISTCMP_CHISQR)
  # print("巴氏距离：%s, 相关性：%s, 卡方：%s" %(match1, match2, match3))
  print("相关性评分：%s分" %(int(match2 * 100)))


def getHistResult(image1, image2) :
  # cv.imwrite('compare-1',create_rgb_hist(src1) )
  # cv.imwrite('compare-2',create_rgb_hist(src2) )
  # plt.subplot(1,2,1)
  # plt.title("diff1")
  # plt.plot(create_rgb_hist(src1))
  # plt.subplot(1,2,2)
  # plt.title("diff2")
  # plt.plot(create_rgb_hist(src2))
  hist_compare(image1, image2)
  # plt.show()
  # cv.waitKey(0)
  # while True: 
  #   #do your thing 
  #   if cv.waitKey(0) & 0xFF == ord('q'): 
  #     break #this loop will break if you press 'q', else it'll wait 
  # cv.destroyAllWindows()
